@@ -71,3 +71,22 @@ class TestDiskAdapter(test.TestCase):
                          self.st_adpt._get_disk_name('type', inst, short=True))
         mock_san.assert_called_with('a_name_t_0123', prefix='t_',
                                     max_len=pvm_const.MaxLen.VDISK_NAME)
+
+    @mock.patch("pypowervm.util.sanitize_file_name_for_api")
+    def test_get_name_by_uuid(self, mock_san):
+        uuid = '01234567-abcd-abcd-abcd-123412341234'
+
+        # Long
+        self.assertEqual(mock_san.return_value,
+                         self.st_adpt.get_name_by_uuid('type', uuid))
+        mock_san.assert_called_with(uuid, prefix='type_',
+                                    max_len=pvm_const.MaxLen.FILENAME_DEFAULT)
+
+        mock_san.reset_mock()
+
+        # Short
+        self.assertEqual(mock_san.return_value,
+                         self.st_adpt.get_name_by_uuid('type', uuid,
+                                                       short=True))
+        mock_san.assert_called_with(uuid, prefix='t_',
+                                    max_len=pvm_const.MaxLen.VDISK_NAME)
