@@ -23,14 +23,12 @@ from nova import context as ctx
 from nova import exception
 from nova import image
 from nova import objects
-from nova.objects import flavor as flavor_obj
 from nova import utils as n_utils
 from nova.virt import configdrive
 from nova.virt import driver
 from oslo_log import log as logging
 from oslo_utils import importutils
 import six
-from taskflow import engines as tf_eng
 from taskflow.patterns import linear_flow as tf_lf
 import time
 
@@ -39,14 +37,14 @@ from pypowervm import const as pvm_const
 from pypowervm import exceptions as pvm_exc
 from pypowervm.helpers import log_helper as log_hlp
 from pypowervm.helpers import vios_busy as vio_hlp
+from pypowervm.tasks import cna as pvm_cna
 from pypowervm.tasks import memory as pvm_mem
 from pypowervm.tasks import partition as pvm_par
-from pypowervm.tasks import power as pvm_pwr
+from pypowervm.tasks import power_opts as pvm_popts
 from pypowervm.tasks import scsi_mapper as pvm_smap
 from pypowervm.tasks import storage as pvm_stor
 from pypowervm.tasks import vterm as pvm_vterm
 from pypowervm import util as pvm_util
-from pypowervm.wrappers import base_partition as pvm_bp
 from pypowervm.wrappers import managed_system as pvm_ms
 
 from nova_powervm import conf as cfg
@@ -54,13 +52,12 @@ from nova_powervm.virt.powervm.disk import driver as disk_dvr
 from nova_powervm.virt.powervm import event
 from nova_powervm.virt.powervm import host as pvm_host
 from nova_powervm.virt.powervm.i18n import _
-from nova_powervm.virt.powervm.i18n import _LE
-from nova_powervm.virt.powervm.i18n import _LI
-from nova_powervm.virt.powervm.i18n import _LW
 from nova_powervm.virt.powervm import image as img
 from nova_powervm.virt.powervm import live_migration as lpm
+from nova_powervm.virt.powervm import media
 from nova_powervm.virt.powervm.nvram import manager as nvram_manager
 from nova_powervm.virt.powervm import slot
+from nova_powervm.virt.powervm.tasks import base as tf_base
 from nova_powervm.virt.powervm.tasks import image as tf_img
 from nova_powervm.virt.powervm.tasks import network as tf_net
 from nova_powervm.virt.powervm.tasks import slot as tf_slot
@@ -69,6 +66,7 @@ from nova_powervm.virt.powervm.tasks import vm as tf_vm
 from nova_powervm.virt.powervm import vm
 from nova_powervm.virt.powervm import volume as vol_attach
 
+
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
@@ -122,7 +120,7 @@ class PowerVMDriver(driver.ComputeDriver):
         pvm_par.validate_vios_ready(self.adapter)
 
         # Do a scrub of the I/O plane to make sure the system is in good shape
-        LOG.info(_LI("Clearing stale I/O connections on driver init."))
+        LOG.info("Clearing stale I/O connections on driver init.")
         pvm_stor.ComprehensiveScrub(self.adapter).execute()
 
         # Initialize the disk adapter.  Sets self.disk_dvr
@@ -140,7 +138,10 @@ class PowerVMDriver(driver.ComputeDriver):
         # Value: overhead (int MB)
         self._inst_overhead_cache = {}
 
-        LOG.info(_LI("The compute driver has been initialized."))
+        # Clean-up any orphan adapters
+        self._cleanup_orphan_adapters(CONF.powervm.pvm_vswitch_for_novalink_io)
+
+        LOG.info("The compute driver has been initialized.")
 
     def cleanup_host(self, host):
         """Clean up anything that is necessary for the driver gracefully stop.
@@ -153,7 +154,7 @@ class PowerVMDriver(driver.ComputeDriver):
         except Exception:
             pass
 
-        LOG.info(_LI("The compute driver has been shutdown."))
+        LOG.info("The compute driver has been shutdown.")
 
     def _get_adapter(self):
         # Build the adapter.  May need to attempt the connection multiple times
@@ -171,6 +172,10 @@ class PowerVMDriver(driver.ComputeDriver):
         self.disk_dvr = importutils.import_object_ns(
             DISK_ADPT_NS, DISK_ADPT_MAPPINGS[CONF.powervm.disk_driver.lower()],
             self.adapter, self.host_uuid)
+        self.capabilities.update(self.disk_dvr.capabilities)
+
+    def manage_image_cache(self, context, all_instances):
+        self.disk_dvr.manage_image_cache(context, all_instances)
 
     def _setup_rebuild_store(self):
         """Set up the store for remote restart objects."""
@@ -206,16 +211,15 @@ class PowerVMDriver(driver.ComputeDriver):
                 _("Expected exactly one host; found %d"), len(syswraps))
         self.host_wrapper = syswraps[0]
         self.host_uuid = self.host_wrapper.uuid
-        LOG.info(_LI("Host UUID is:%s"), self.host_uuid)
+        LOG.info("Host UUID is:%s", self.host_uuid)
 
     @staticmethod
     def _log_operation(op, instance):
         """Log entry point of driver operations."""
-        LOG.info(_LI('Operation: %(op)s. Virtual machine display name: '
-                     '%(display_name)s, name: %(name)s'),
+        LOG.info('Operation: %(op)s. Virtual machine display name: '
+                 '%(display_name)s, name: %(name)s',
                  {'op': op, 'display_name': instance.display_name,
-                  'name': instance.name},
-                 instance=instance)
+                  'name': instance.name}, instance=instance)
 
     def get_info(self, instance):
         """Get the current status of an instance, by name (not ID!)
@@ -279,8 +283,8 @@ class PowerVMDriver(driver.ComputeDriver):
                     self.adapter, self.host_uuid, {'max_mem': max_mem})
                 self._inst_overhead_cache[max_mem] = overhead
 
-        except Exception as e:
-            LOG.exception(e)
+        except Exception:
+            LOG.exception("PowerVM error estimating instance overhead.")
         finally:
             return {'memory_mb': overhead}
 
@@ -328,15 +332,15 @@ class PowerVMDriver(driver.ComputeDriver):
                     LOG.debug('Disks found on shared storage.',
                               instance=instance)
                     return True
-            except Exception as e:
-                LOG.exception(e)
+            except Exception:
+                LOG.exception("PowerVM error checking instance on disk.",
+                              instance=instance)
 
         LOG.debug('Instance disks not found on this host.', instance=instance)
         return False
 
     def spawn(self, context, instance, image_meta, injected_files,
-              admin_password, network_info=None, block_device_info=None,
-              flavor=None):
+              admin_password, network_info=None, block_device_info=None):
         """Create a new instance/VM/domain on the virtualization platform.
 
         Once this successfully completes, the instance should be
@@ -346,31 +350,19 @@ class PowerVMDriver(driver.ComputeDriver):
         cleaned up, and the virtualization platform should be in the state
         that it was before this call began.
 
-        Spawn can be called while deploying an instance for the first time or
-        it can be called to recreate an instance that was shelved or during
-        evacuation.  We have to be careful to handle all these cases.  During
-        evacuation, when on shared storage, the image_meta will be empty.
-
         :param context: security context
-        :param instance: Instance object as returned by DB layer.
+        :param instance: nova.objects.instance.Instance
                          This function should use the data there to guide
                          the creation of the new instance.
         :param nova.objects.ImageMeta image_meta:
             The metadata of the image of the instance.
         :param injected_files: User files to inject into instance.
         :param admin_password: Administrator password to set in instance.
-        :param network_info:
-           :py:meth:`~nova.network.manager.NetworkManager.get_instance_nw_info`
+        :param network_info: instance network information
         :param block_device_info: Information about block devices to be
                                   attached to the instance.
-        :param flavor: The flavor for the instance to be spawned.
         """
         self._log_operation('spawn', instance)
-        if not flavor:
-            admin_ctx = ctx.get_admin_context(read_deleted='yes')
-            flavor = (
-                flavor_obj.Flavor.get_by_id(admin_ctx,
-                                            instance.instance_type_id))
 
         # Extract the block devices.
         bdms = self._extract_bdm(block_device_info)
@@ -405,7 +397,7 @@ class PowerVMDriver(driver.ComputeDriver):
         # Create task produce a FeedTask that will be used to scrub stale
         # adapters immediately after the LPAR is created.
         flow_spawn.add(tf_vm.Create(
-            self.adapter, self.host_wrapper, instance, flavor,
+            self.adapter, self.host_wrapper, instance,
             stg_ftsk=(None if recreate else stg_ftsk), nvram_mgr=nvram_mgr,
             slot_mgr=slot_mgr))
 
@@ -426,11 +418,10 @@ class PowerVMDriver(driver.ComputeDriver):
             else:
                 # Creates the boot image.
                 flow_spawn.add(tf_stg.CreateDiskForImg(
-                    self.disk_dvr, context, instance, image_meta,
-                    disk_size=flavor.root_gb))
+                    self.disk_dvr, context, instance, image_meta))
             # Connects up the disk to the LPAR
-            flow_spawn.add(tf_stg.ConnectDisk(self.disk_dvr, context, instance,
-                                              stg_ftsk=stg_ftsk))
+            flow_spawn.add(tf_stg.ConnectDisk(
+                self.disk_dvr, instance, stg_ftsk=stg_ftsk))
 
         # Determine if there are volumes to connect.  If so, add a connection
         # for each type.
@@ -459,11 +450,22 @@ class PowerVMDriver(driver.ComputeDriver):
         # Save the slot map information
         flow_spawn.add(tf_slot.SaveSlotStore(instance, slot_mgr))
 
+        pwr_opts = pvm_popts.PowerOnOpts()
+        if CONF.powervm.remove_vopt_media_on_boot:
+            # Get the cfgdrive media name for the vopt removal task in PowerOn.
+            media_name = pvm_util.sanitize_file_name_for_api(
+                instance.name, prefix=media.CFG_DRV_PREFIX,
+                suffix=media.CFG_DRV_SUFFIX,
+                max_len=pvm_const.MaxLen.VOPT_NAME)
+            pwr_opts.remove_optical(
+                media_name, time=CONF.powervm.remove_vopt_media_time)
+
         # Last step is to power on the system.
-        flow_spawn.add(tf_vm.PowerOn(self.adapter, self.host_uuid, instance))
+        flow_spawn.add(tf_vm.PowerOn(self.adapter, instance,
+                                     pwr_opts=pwr_opts))
 
         # Run the flow.
-        tf_eng.run(flow_spawn)
+        tf_base.run(flow_spawn, instance=instance)
 
     def _add_volume_connection_tasks(self, context, instance, bdms,
                                      flow, stg_ftsk, slot_mgr):
@@ -568,8 +570,7 @@ class PowerVMDriver(driver.ComputeDriver):
             if shutdown:
                 # Power Off the LPAR. If its disks are about to be deleted,
                 # VSP hard shutdown it.
-                flow.add(tf_vm.PowerOff(self.adapter, self.host_uuid,
-                                        pvm_inst_uuid, instance,
+                flow.add(tf_vm.PowerOff(self.adapter, instance,
                                         force_immediate=destroy_disks))
 
             # Create the transaction manager (FeedTask) for Storage I/O.
@@ -604,12 +605,12 @@ class PowerVMDriver(driver.ComputeDriver):
             if not self._is_booted_from_volume(block_device_info):
                 # Detach the disk storage adapters (when the stg_ftsk runs)
                 flow.add(tf_stg.DetachDisk(
-                    self.disk_dvr, context, instance, stg_ftsk))
+                    self.disk_dvr, instance, stg_ftsk=stg_ftsk))
 
                 # Delete the storage disks
                 if destroy_disks:
                     destroy_disk_task = tf_stg.DeleteDisk(
-                        self.disk_dvr, context, instance)
+                        self.disk_dvr, instance)
 
             # It's possible that volume disconnection may have failed for disks
             # which had been removed from the VIOS by the storage back end
@@ -620,12 +621,12 @@ class PowerVMDriver(driver.ComputeDriver):
             def _rm_vscsi_maps(vwrap):
                 removals = pvm_smap.remove_maps(vwrap, pvm_inst_uuid)
                 if removals:
-                    LOG.warning(_LW("Removing %(num_maps)d storage-less VSCSI "
-                                    "mappings associated with LPAR ID "
-                                    "%(lpar_uuid)s from VIOS %(vios_name)s."),
+                    LOG.warning("Removing %(num_maps)d storage-less VSCSI "
+                                "mappings associated with LPAR ID "
+                                "%(lpar_uuid)s from VIOS %(vios_name)s.",
                                 {'num_maps': len(removals),
                                  'lpar_uuid': pvm_inst_uuid,
-                                 'vios_name': vwrap.name})
+                                 'vios_name': vwrap.name}, instance=instance)
                 return removals
             stg_ftsk.add_functor_subtask(_rm_vscsi_maps)
 
@@ -642,7 +643,7 @@ class PowerVMDriver(driver.ComputeDriver):
             # the NVRAM from the store.
             # Note: If moving to a Graph Flow, will need to change to depend on
             # the prior step.
-            flow.add(tf_vm.Delete(self.adapter, pvm_inst_uuid, instance))
+            flow.add(tf_vm.Delete(self.adapter, instance))
 
             if (destroy_disks and
                     instance.vm_state not in KEEP_NVRAM_STATES and
@@ -653,18 +654,19 @@ class PowerVMDriver(driver.ComputeDriver):
                 flow.add(tf_vm.DeleteNvram(self.nvram_mgr, instance))
                 flow.add(tf_slot.DeleteSlotStore(instance, slot_mgr))
 
-            # Build the engine & run!
-            tf_eng.run(flow)
+            # Run the flow
+            tf_base.run(flow, instance=instance)
 
         try:
             pvm_inst_uuid = vm.get_pvm_uuid(instance)
             _setup_flow_and_run()
         except exception.InstanceNotFound:
-            LOG.warning(_LW('VM was not found during destroy operation.'),
+            LOG.warning('VM was not found during destroy operation.',
                         instance=instance)
             return
         except Exception as e:
-                LOG.exception(e)
+                LOG.exception("PowerVM error destroying instance.",
+                              instance=instance)
                 # Convert to a Nova exception
                 raise exception.InstanceTerminationFailure(
                     reason=six.text_type(e))
@@ -687,7 +689,7 @@ class PowerVMDriver(driver.ComputeDriver):
         :param migrate_data: a LiveMigrateData object
         """
         if instance.task_state == task_states.RESIZE_REVERTING:
-            LOG.info(_LI('Destroy called for migrated/resized instance.'),
+            LOG.info('Destroy called for migrated/resized instance.',
                      instance=instance)
             # This destroy is part of resize or migrate.  It's called to
             # revert the resize/migration on the destination host.
@@ -700,7 +702,7 @@ class PowerVMDriver(driver.ComputeDriver):
                                    qprop='PartitionName', log_errors=False)
             if vm_name == self._gen_resize_name(instance, same_host=True):
                 # Since it matches it must have been a resize, don't delete it!
-                LOG.info(_LI('Ignoring destroy call during resize revert.'),
+                LOG.info('Ignoring destroy call during resize revert.',
                          instance=instance)
                 return
 
@@ -732,9 +734,8 @@ class PowerVMDriver(driver.ComputeDriver):
         # Save the new slot info
         flow.add(tf_slot.SaveSlotStore(instance, slot_mgr))
 
-        # Build the engine & run!
-        engine = tf_eng.load(flow)
-        engine.run()
+        # Run the flow
+        tf_base.run(flow, instance=instance)
 
         # The volume connector may have updated the system metadata.  Save
         # the instance to persist the data.  Spawn/destroy auto saves instance,
@@ -758,8 +759,8 @@ class PowerVMDriver(driver.ComputeDriver):
         # host.  If the migration failed, then the VM is probably not on
         # the destination host.
         if not vm.instance_exists(self.adapter, instance):
-            LOG.info(_LI('During volume detach, the instance was not found'
-                         ' on this host.'), instance=instance)
+            LOG.info('During volume detach, the instance was not found on '
+                     'this host.', instance=instance)
 
             # Check if there is live migration cleanup to do on this volume.
             mig = self.live_migrations.get(instance.uuid, None)
@@ -777,9 +778,8 @@ class PowerVMDriver(driver.ComputeDriver):
         # Save the new slot info
         flow.add(tf_slot.SaveSlotStore(instance, slot_mgr))
 
-        # Build the engine & run!
-        engine = tf_eng.load(flow)
-        engine.run()
+        # Run the flow
+        tf_base.run(flow, instance=instance)
 
     def snapshot(self, context, instance, image_id, update_task_state):
         """Snapshots the specified instance.
@@ -826,8 +826,8 @@ class PowerVMDriver(driver.ComputeDriver):
         # device
         flow.add(tf_stg.RemoveInstanceDiskFromMgmt(self.disk_dvr, instance))
 
-        # Build the engine & run
-        tf_eng.load(flow).run()
+        # Run the flow
+        tf_base.run(flow, instance=instance)
 
     def rescue(self, context, instance, network_info, image_meta,
                rescue_password):
@@ -845,33 +845,27 @@ class PowerVMDriver(driver.ComputeDriver):
         """
         self._log_operation('rescue', instance)
 
-        pvm_inst_uuid = vm.get_pvm_uuid(instance)
         # Define the flow
         flow = tf_lf.Flow("rescue")
 
-        # Get the LPAR Wrapper
-        flow.add(tf_vm.Get(self.adapter, self.host_uuid, instance))
-
         # Power Off the LPAR
-        flow.add(tf_vm.PowerOff(self.adapter, self.host_uuid,
-                                pvm_inst_uuid, instance))
+        flow.add(tf_vm.PowerOff(self.adapter, instance))
 
-        # Creates the boot image.
+        # Create and populate the rescue disk.
         flow.add(tf_stg.CreateDiskForImg(
             self.disk_dvr, context, instance, image_meta,
             image_type=disk_dvr.DiskType.RESCUE))
 
-        # Connects up the disk to the LPAR
-        flow.add(tf_stg.ConnectDisk(self.disk_dvr, context, instance))
+        # Connect up the disk to the LPAR
+        flow.add(tf_stg.ConnectDisk(self.disk_dvr, instance))
 
         # Last step is to power on the system.
         flow.add(tf_vm.PowerOn(
-            self.adapter, self.host_uuid, instance,
-            pwr_opts={pvm_pwr.BootMode.KEY: pvm_pwr.BootMode.SMS}))
+            self.adapter, instance,
+            pwr_opts=pvm_popts.PowerOnOpts().bootmode(pvm_popts.BootMode.SMS)))
 
-        # Build the engine & run!
-        engine = tf_eng.load(flow)
-        engine.run()
+        # Run the flow
+        tf_base.run(flow, instance=instance)
 
     def unrescue(self, instance, network_info):
         """Unrescue the specified instance.
@@ -880,32 +874,24 @@ class PowerVMDriver(driver.ComputeDriver):
         """
         self._log_operation('unrescue', instance)
 
-        pvm_inst_uuid = vm.get_pvm_uuid(instance)
-        context = ctx.get_admin_context()
-
         # Define the flow
         flow = tf_lf.Flow("unrescue")
 
-        # Get the LPAR Wrapper
-        flow.add(tf_vm.Get(self.adapter, self.host_uuid, instance))
-
         # Power Off the LPAR
-        flow.add(tf_vm.PowerOff(self.adapter, self.host_uuid,
-                                pvm_inst_uuid, instance))
+        flow.add(tf_vm.PowerOff(self.adapter, instance))
 
         # Detach the disk adapter for the rescue image
-        flow.add(tf_stg.DetachDisk(self.disk_dvr, context, instance,
-                                   disk_type=[disk_dvr.DiskType.RESCUE]))
+        flow.add(tf_stg.DetachDisk(
+            self.disk_dvr, instance, disk_type=[disk_dvr.DiskType.RESCUE]))
 
         # Delete the storage disk for the rescue image
-        flow.add(tf_stg.DeleteDisk(self.disk_dvr, context, instance))
+        flow.add(tf_stg.DeleteDisk(self.disk_dvr, instance))
 
         # Last step is to power on the system.
-        flow.add(tf_vm.PowerOn(self.adapter, self.host_uuid, instance))
+        flow.add(tf_vm.PowerOn(self.adapter, instance))
 
-        # Build the engine & run!
-        engine = tf_eng.load(flow)
-        engine.run()
+        # Run the flow
+        tf_base.run(flow, instance=instance)
 
     def power_off(self, instance, timeout=0, retry_interval=0):
         """Power off the specified instance.
@@ -918,8 +904,8 @@ class PowerVMDriver(driver.ComputeDriver):
         self._log_operation('power_off', instance)
         force_immediate = (timeout == 0)
         timeout = timeout or None
-        vm.power_off(self.adapter, instance, self.host_uuid,
-                     force_immediate=force_immediate, timeout=timeout)
+        vm.power_off(self.adapter, instance, force_immediate=force_immediate,
+                     timeout=timeout)
 
     def power_on(self, context, instance, network_info,
                  block_device_info=None):
@@ -928,7 +914,7 @@ class PowerVMDriver(driver.ComputeDriver):
         :param instance: nova.objects.instance.Instance
         """
         self._log_operation('power_on', instance)
-        vm.power_on(self.adapter, instance, self.host_uuid)
+        vm.power_on(self.adapter, instance)
 
     def reboot(self, context, instance, network_info, reboot_type,
                block_device_info=None, bad_volumes_callback=None):
@@ -949,19 +935,8 @@ class PowerVMDriver(driver.ComputeDriver):
             encountered
         """
         self._log_operation(reboot_type + ' reboot', instance)
-        force_immediate = reboot_type == 'HARD'
-        entry = vm.get_instance_wrapper(self.adapter, instance)
-        if entry.state != pvm_bp.LPARState.NOT_ACTIVATED:
-            pvm_pwr.power_off(entry, self.host_uuid, restart=True,
-                              force_immediate=force_immediate)
-        else:
-            # pypowervm does NOT throw an exception if "already down".
-            # Any other exception from pypowervm is a legitimate failure;
-            # let it raise up.
-            # If we get here, pypowervm thinks the instance is down.
-            pvm_pwr.power_on(entry, self.host_uuid)
-
-        # Again, pypowervm exceptions are sufficient to indicate real failure.
+        vm.reboot(self.adapter, instance, reboot_type == 'HARD')
+        # pypowervm exceptions are sufficient to indicate real failure.
         # Otherwise, pypowervm thinks the instance is up.
         return True
 
@@ -1019,16 +994,15 @@ class PowerVMDriver(driver.ComputeDriver):
         # Save the new slot info
         flow.add(tf_slot.SaveSlotStore(instance, slot_mgr))
 
-        # Build the engine & run!
-        engine = tf_eng.load(flow)
+        # Run the flow
         try:
-            engine.run()
+            tf_base.run(flow, instance=instance)
         except exception.InstanceNotFound:
             raise exception.VirtualInterfacePlugException(
                 _("Plug vif failed because instance %s was not found.")
                 % instance.name)
-        except Exception as e:
-            LOG.exception(e)
+        except Exception:
+            LOG.exception("PowerVM error plugging vifs.", instance=instance)
             raise exception.VirtualInterfacePlugException(
                 _("Plug vif failed because of an unexpected error."))
 
@@ -1050,17 +1024,16 @@ class PowerVMDriver(driver.ComputeDriver):
         # Save the new slot info
         flow.add(tf_slot.SaveSlotStore(instance, slot_mgr))
 
-        # Build the engine & run!
-        engine = tf_eng.load(flow)
+        # Run the flow
         try:
-            engine.run()
-        except exception.InstanceNotFound as ei:
-            LOG.exception(ei)
-            LOG.warning(_LW('VM was not found during unplug operation '
-                            'as it is already possibly deleted'),
-                        instance=instance)
-        except Exception as e:
-            LOG.exception(e)
+            tf_base.run(flow, instance=instance)
+        except exception.InstanceNotFound:
+            LOG.exception('VM was not found during unplug operation '
+                          'as it is already possibly deleted.',
+                          instance=instance)
+        except Exception:
+            LOG.exception("PowerVM error trying to unplug vifs.",
+                          instance=instance)
             raise exception.InterfaceDetachFailed(instance_uuid=instance.uuid)
 
     def get_available_nodes(self, refresh=False):
@@ -1072,7 +1045,7 @@ class PowerVMDriver(driver.ComputeDriver):
         [hypervisor_hostname].
         """
 
-        return [self.host_wrapper.mtms.mtms_str]
+        return [CONF.host]
 
     def legacy_nwinfo(self):
         """Indicate if the driver requires the legacy network_info format."""
@@ -1083,8 +1056,8 @@ class PowerVMDriver(driver.ComputeDriver):
         # This code was pulled from the libvirt driver.
         ips = compute_utils.get_machine_ips()
         if CONF.my_ip not in ips:
-            LOG.warning(_LW('my_ip address (%(my_ip)s) was not found on '
-                            'any of the interfaces: %(ifaces)s'),
+            LOG.warning('my_ip address (%(my_ip)s) was not found on '
+                        'any of the interfaces: %(ifaces)s',
                         {'my_ip': CONF.my_ip, 'ifaces': ", ".join(ips)})
         return CONF.my_ip
 
@@ -1140,14 +1113,11 @@ class PowerVMDriver(driver.ComputeDriver):
                 # Get disk info from disk driver.
                 disk_info = dict(disk_info, **self.disk_dvr.get_info())
 
-        pvm_inst_uuid = vm.get_pvm_uuid(instance)
-
         # Define the migrate flow
         flow = tf_lf.Flow("migrate_vm")
 
         # Power off the VM
-        flow.add(tf_vm.PowerOff(self.adapter, self.host_uuid,
-                                pvm_inst_uuid, instance))
+        flow.add(tf_vm.PowerOff(self.adapter, instance))
 
         if not same_host:
             # If VM is moving to a new host make sure the NVRAM is at the very
@@ -1156,8 +1126,8 @@ class PowerVMDriver(driver.ComputeDriver):
                      immediate=True))
         if flavor.root_gb > instance.root_gb:
             # Resize the root disk
-            flow.add(tf_stg.ExtendDisk(self.disk_dvr, context, instance,
-                                       dict(type='boot'), flavor.root_gb))
+            flow.add(tf_stg.ExtendDisk(
+                self.disk_dvr, instance, dict(type='boot'), flavor.root_gb))
 
         # Disconnect any volumes that are attached.  They are reattached
         # on the new VM (or existing VM if this is just a resize.)
@@ -1192,7 +1162,7 @@ class PowerVMDriver(driver.ComputeDriver):
         new_name = self._gen_resize_name(instance, same_host=same_host)
         flow.add(tf_vm.Rename(self.adapter, instance, new_name))
         try:
-            tf_eng.run(flow)
+            tf_base.run(flow, instance=instance)
         except Exception as e:
             raise exception.InstanceFaultRollback(e)
 
@@ -1279,13 +1249,13 @@ class PowerVMDriver(driver.ComputeDriver):
             # This is just a resize.
             new_name = self._gen_resize_name(instance, same_host=True)
             flow.add(tf_vm.Resize(self.adapter, self.host_wrapper, instance,
-                                  instance.flavor, name=new_name))
+                                  name=new_name))
         else:
             # This is a migration over to another host.  We have a lot of work.
             # Create the LPAR
             flow.add(tf_vm.Create(self.adapter, self.host_wrapper, instance,
-                                  instance.flavor, stg_ftsk,
-                                  nvram_mgr=self.nvram_mgr, slot_mgr=slot_mgr))
+                                  stg_ftsk=stg_ftsk, nvram_mgr=self.nvram_mgr,
+                                  slot_mgr=slot_mgr))
 
             # Create a flow for the network IO
             flow.add(tf_net.PlugVifs(self.virtapi, self.adapter, instance,
@@ -1300,8 +1270,8 @@ class PowerVMDriver(driver.ComputeDriver):
 
                 # Connects up the disk to the LPAR
                 # TODO(manas) Connect the disk flow into the slot lookup map
-                flow.add(tf_stg.ConnectDisk(self.disk_dvr, context, instance,
-                                            stg_ftsk=stg_ftsk))
+                flow.add(tf_stg.ConnectDisk(
+                    self.disk_dvr, instance, stg_ftsk=stg_ftsk))
 
         if bdms:
             # Determine if there are volumes to connect.  If so, add a
@@ -1315,12 +1285,11 @@ class PowerVMDriver(driver.ComputeDriver):
             flow.add(stg_ftsk)
 
         if power_on:
-            # Get the lpar wrapper (required by power-on), then power-on
-            flow.add(tf_vm.Get(self.adapter, self.host_uuid, instance))
-            flow.add(tf_vm.PowerOn(self.adapter, self.host_uuid, instance))
+            flow.add(tf_vm.PowerOn(self.adapter, instance))
 
+        # Run the flow
         try:
-            tf_eng.run(flow)
+            tf_base.run(flow, instance=instance)
         except Exception as e:
             raise exception.InstanceFaultRollback(e)
 
@@ -1369,12 +1338,11 @@ class PowerVMDriver(driver.ComputeDriver):
         # makes it easy to handle both resize and migrate.
         #
         # The flavor should be the 'old' flavor now.
-        vm.power_off(self.adapter, instance, self.host_uuid)
-        vm.update(self.adapter, self.host_wrapper, instance,
-                  instance.flavor)
+        vm.power_off(self.adapter, instance)
+        vm.update(self.adapter, self.host_wrapper, instance)
 
         if power_on:
-            vm.power_on(self.adapter, instance, self.host_uuid)
+            vm.power_on(self.adapter, instance)
 
     def ensure_filtering_rules_for_instance(self, instance, network_info):
         """Setting up filtering rules and waiting for its completion.
@@ -1420,8 +1388,8 @@ class PowerVMDriver(driver.ComputeDriver):
 
         :returns: a dict containing migration info (hypervisor-dependent)
         """
-        LOG.info(_LI("Checking live migration capability on destination "
-                     "host."), instance=instance)
+        LOG.info("Checking live migration capability on destination host.",
+                 instance=instance)
 
         mig = lpm.LiveMigrationDest(self, instance)
         self.live_migrations[instance.uuid] = mig
@@ -1435,8 +1403,8 @@ class PowerVMDriver(driver.ComputeDriver):
         :param context: security context
         :param dest_check_data: result of check_can_live_migrate_destination
         """
-        LOG.info(_LI("Cleaning up from checking live migration capability "
-                     "on destination."))
+        LOG.info("Cleaning up from checking live migration capability "
+                 "on destination.")
 
     def check_can_live_migrate_source(self, context, instance,
                                       dest_check_data, block_device_info=None):
@@ -1451,7 +1419,7 @@ class PowerVMDriver(driver.ComputeDriver):
         :param block_device_info: result of _get_instance_block_device_info
         :returns: a dict containing migration info (hypervisor-dependent)
         """
-        LOG.info(_LI("Checking live migration capability on source host."),
+        LOG.info("Checking live migration capability on source host.",
                  instance=instance)
         mig = lpm.LiveMigrationSrc(self, instance, dest_check_data)
         self.live_migrations[instance.uuid] = mig
@@ -1473,8 +1441,7 @@ class PowerVMDriver(driver.ComputeDriver):
         :param disk_info: instance disk information
         :param migrate_data: a LiveMigrateData object
         """
-        LOG.info(_LI("Pre live migration processing."),
-                 instance=instance)
+        LOG.info("Pre live migration processing.", instance=instance)
         mig = self.live_migrations[instance.uuid]
 
         # Get a volume driver for each volume
@@ -1512,7 +1479,7 @@ class PowerVMDriver(driver.ComputeDriver):
                 mig.live_migration(context, migrate_data)
             except pvm_exc.JobRequestTimedOut as timeout_ex:
                 # If the migration operation exceeds configured timeout
-                LOG.error(_LE("Live migration timed out. Aborting migration"),
+                LOG.error("Live migration timed out. Aborting migration",
                           instance=instance)
                 mig.migration_abort()
                 self._migration_exception_util(context, instance, dest,
@@ -1520,7 +1487,8 @@ class PowerVMDriver(driver.ComputeDriver):
                                                migrate_data,
                                                mig, ex=timeout_ex)
             except Exception as e:
-                LOG.exception(e)
+                LOG.exception("PowerVM error during live migration.",
+                              instance=instance)
                 self._migration_exception_util(context, instance, dest,
                                                recover_method,
                                                migrate_data,
@@ -1550,12 +1518,13 @@ class PowerVMDriver(driver.ComputeDriver):
         :param ex: exception reason
 
         """
-        LOG.warning(_LW("Rolling back live migration."), instance=instance)
+        LOG.warning("Rolling back live migration.", instance=instance)
         try:
             mig.rollback_live_migration(context)
             recover_method(context, instance, dest, migrate_data=migrate_data)
-        except Exception as e:
-            LOG.exception(e)
+        except Exception:
+            LOG.exception("PowerVM error rolling back live migration.",
+                          instance=instance)
 
         raise lpm.LiveMigrationFailed(name=instance.name,
                                       reason=six.text_type(ex))
@@ -1641,7 +1610,7 @@ class PowerVMDriver(driver.ComputeDriver):
         :param instance: instance object reference
         :param network_info: instance network information
         """
-        LOG.info(_LI("Post live migration processing on source host."),
+        LOG.info("Post live migration processing on source host.",
                  instance=instance)
         mig = self.live_migrations[instance.uuid]
         mig.post_live_migration_at_source(network_info)
@@ -1657,7 +1626,7 @@ class PowerVMDriver(driver.ComputeDriver):
         :param network_info: instance network information
         :param block_migration: if true, post operation of block_migration.
         """
-        LOG.info(_LI("Post live migration processing on destination host."),
+        LOG.info("Post live migration processing on destination host.",
                  instance=instance)
         mig = self.live_migrations[instance.uuid]
         mig.instance = instance
@@ -1763,16 +1732,12 @@ class PowerVMDriver(driver.ComputeDriver):
                 self.adapter, lpar_uuid, host, vnc_path=lpar_uuid,
                 use_x509_auth=use_x509_auth, ca_certs=ca_certs,
                 server_cert=server_cert, server_key=server_key)
-        except Exception as err:
-            # If the LPAR was not found, then give a more descriptive message
-            if isinstance(err, pvm_exc.HttpError):
-                if err.response.status == 404:
-                    msg = _("Unable to open console since virtual machine "
-                            "%s does not exist.") % instance['display_name']
-                    raise exception.ConsoleTypeUnavailable(message=msg)
+        except pvm_exc.HttpNotFound:
+            raise exception.InstanceNotFound(instance_id=instance.uuid)
+        except pvm_exc.Error:
             # Otherwise wrapper the error in an exception that can be handled
-            raise exception.ConsoleTypeUnavailable(
-                message=_("Unable to open console.  Error is: %s") % err)
+            LOG.exception("Unable to open console.", instance=instance)
+            raise exception.InternalError(err=_("Unable to open console."))
 
         # Note that the VNC viewer will wrap the internal_access_path with
         # the HTTP content.
@@ -1808,8 +1773,8 @@ class PowerVMDriver(driver.ComputeDriver):
             vol_cls = vol_attach.get_volume_class(driver_type)
             xags.update(set(vol_cls.min_xags()))
 
-        LOG.debug('Instance XAGs for VM %(inst)s is %(xags)s.',
-                  {'inst': instance.name, 'xags': ','.join(xags)})
+        LOG.debug('Instance XAGs: %(xags)s.', {'xags': ','.join(xags)},
+                  instance=instance)
         return list(xags)
 
     def _get_boot_connectivity_type(self, context, bdms, block_device_info):
@@ -1832,7 +1797,8 @@ class PowerVMDriver(driver.ComputeDriver):
         else:
             return boot_conn_type
 
-    def _get_connectivity_type(self, bdm):
+    @staticmethod
+    def _get_connectivity_type(bdm):
         conn_info = bdm.get('connection_info')
         if 'connection-type' in conn_info['data']:
             connectivity_type = conn_info['data']['connection-type']
@@ -1849,3 +1815,10 @@ class PowerVMDriver(driver.ComputeDriver):
         :returns: Boolean value. If True deallocate networks on reschedule.
         """
         return True
+
+    def _cleanup_orphan_adapters(self, vswitch_name):
+        """Finds and removes trunk VEAs that have no corresponding CNA."""
+        orphans = pvm_cna.find_orphaned_trunks(self.adapter, vswitch_name)
+        for orphan in orphans:
+            LOG.info("Deleting orphan CNA: %s", orphan.dev_name)
+            orphan.delete()
