@@ -1,4 +1,4 @@
-# Copyright 2016 IBM Corp.
+# Copyright 2016, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -23,7 +23,7 @@ from pypowervm.tasks import slot_map
 from pypowervm.tasks import storage as pvm_tstor
 
 from nova_powervm.virt.powervm import exception as p_exc
-from nova_powervm.virt.powervm.i18n import _LW
+
 
 LOG = logging.getLogger(__name__)
 
@@ -124,7 +124,7 @@ class NovaSlotManager(slot_map.SlotMapStore):
         pv_vscsi_vol_to_vio = {}
         fabric_names = []
         for bdm, vol_drv in vol_drv_iter:
-            if vol_drv.vol_type() == 'vscsi':
+            if vol_drv.vol_type() in ['vscsi', 'fileio']:
                 self._pv_vscsi_vol_to_vio(vol_drv, pv_vscsi_vol_to_vio)
             elif len(fabric_names) == 0 and vol_drv.vol_type() == 'npiv':
                 fabric_names = vol_drv._fabric_names()
@@ -177,12 +177,11 @@ class SwiftSlotManager(NovaSlotManager):
         try:
             self.store_api.delete_slot_map(key)
         except Exception:
-            LOG.warning(_LW("Unable to delete the slot map from Swift backing "
-                            "store with ID %(key)s.  Will require "
-                            "manual cleanup."), {'key': key}, self.instance)
+            LOG.warning("Unable to delete the slot map from Swift backing "
+                        "store with ID %(key)s.  Will require manual cleanup.",
+                        {'key': key}, instance=self.instance)
 
 
 class NoopSlotManager(NovaSlotManager):
     """No op Slot Map (for when Swift is not used - which is standard)."""
-
     pass
