@@ -1,4 +1,4 @@
-# Copyright 2015, 2016 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -18,9 +18,6 @@ from oslo_log import log as logging
 
 from nova_powervm import conf as cfg
 from nova_powervm.virt.powervm import exception as p_exc
-from nova_powervm.virt.powervm.i18n import _LE
-from nova_powervm.virt.powervm.i18n import _LI
-from nova_powervm.virt.powervm.i18n import _LW
 from nova_powervm.virt.powervm import vm
 from nova_powervm.virt.powervm.volume import driver as v_driver
 from nova_powervm.virt.powervm.volume import volume as volume
@@ -33,6 +30,7 @@ from taskflow import task
 
 import six
 
+
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 DEVNAME_KEY = 'target_devname'
@@ -92,13 +90,15 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
                  not (could be the Virtual I/O Server does not have
                  connectivity to the hdisk).
         """
+        transport_type = self.connection_info["driver_volume_type"]
         host_ip = self.connection_info["data"]["target_portal"]
         iqn = self.connection_info["data"]["target_iqn"]
         password = self.connection_info["data"]["auth_password"]
         user = self.connection_info["data"]["auth_username"]
         target_name = "ISCSI-" + iqn.split(":")[1]
         device_name, udid = hdisk.discover_iscsi(
-            self.adapter, host_ip, user, password, iqn, vios_w.uuid)
+            self.adapter, host_ip, user, password, iqn, vios_w.uuid,
+            transport_type=transport_type)
         slot, lua = slot_mgr.build_map.get_vscsi_slot(vios_w, device_name)
         if device_name is not None and udid is not None:
             device_name = '/dev/' + device_name
@@ -114,7 +114,8 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
             self._set_devname(device_name)
             self._set_udid(udid)
 
-            LOG.debug('Device attached: %s', device_name)
+            LOG.debug('Device attached: %s', device_name,
+                      instance=self.instance)
 
             # Valid attachment
             return True
@@ -139,7 +140,8 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
                      otherwise.
             """
             LOG.debug("Disconnect volume %(vol)s from vios uuid %(uuid)s",
-                      dict(vol=self.volume_id, uuid=vios_w.uuid))
+                      dict(vol=self.volume_id, uuid=vios_w.uuid),
+                      instance=self.instance)
             device_name = None
             try:
                 device_name = self._get_devname()
@@ -150,28 +152,27 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
                     # If we have no device name, at this point
                     # we should not continue.  Subsequent scrub code on future
                     # deploys will clean this up.
-                    LOG.warning(_LW(
+                    LOG.warning(
                         "Disconnect Volume: The backing hdisk for volume "
                         "%(volume_id)s on Virtual I/O Server %(vios)s is "
                         "not in a valid state.  No disconnect "
-                        "actions to be taken as volume is not healthy."),
+                        "actions to be taken as volume is not healthy.",
                         {'volume_id': self.volume_id, 'vios': vios_w.name},
                         instance=self.instance)
                     return False
 
-            except Exception as e:
-                LOG.warning(_LW(
-                    "Disconnect Volume: Failed to find disk on Virtual I/O "
-                    "Server %(vios_name)s for volume %(volume_id)s."
-                    " Error: %(error)s"),
-                    {'error': e, 'vios_name': vios_w.name,
-                     'volume_id': self.volume_id}, instance=self.instance)
+            except Exception:
+                LOG.exception(
+                    "Disconnect Volume: Failed to find device on Virtual I/O "
+                    "Server %(vios_name)s for volume %(volume_id)s.",
+                    {'vios_name': vios_w.name, 'volume_id': self.volume_id},
+                    instance=self.instance)
                 return False
 
             # We have found the device name
-            LOG.info(_LI("Disconnect Volume: Discovered the device %(hdisk)s "
-                         "on Virtual I/O Server %(vios_name)s for volume "
-                         "%(volume_id)s."),
+            LOG.info("Disconnect Volume: Discovered the device %(hdisk)s "
+                     "on Virtual I/O Server %(vios_name)s for volume "
+                     "%(volume_id)s.",
                      {'volume_id': self.volume_id,
                       'vios_name': vios_w.name, 'hdisk': device_name},
                      instance=self.instance)
@@ -203,17 +204,14 @@ class IscsiVolumeAdapter(volume.VscsiVolumeAdapter,
             # Warn if no hdisks disconnected.
             if not any([result['vio_modified']
                         for result in ret['wrapper_task_rets'].values()]):
-                LOG.warning(_LW(
-                    "Disconnect Volume: Failed to disconnect the  volume "
-                    "%(volume_id)s on ANY of the Virtual I/O Servers for "
-                    "instance %(inst)s."),
-                    {'inst': self.instance.name, 'volume_id': self.volume_id},
-                    instance=self.instance)
+                LOG.warning(
+                    "Disconnect Volume: Failed to disconnect the volume "
+                    "%(volume_id)s on ANY of the Virtual I/O Servers.",
+                    {'volume_id': self.volume_id}, instance=self.instance)
 
         except Exception as e:
-            LOG.error(_LE('Cannot detach volumes from virtual machine: %s'),
-                      self.vm_uuid, instance=self.instance)
-            LOG.exception(_LE('Error: %s'), e, instance=self.instance)
+            LOG.exception('PowerVM error detaching volume from virtual '
+                          'machine.', instance=self.instance)
             ex_args = {'volume_id': self.volume_id, 'reason': six.text_type(e),
                        'instance_name': self.instance.name}
             raise p_exc.VolumeDetachFailed(**ex_args)
